#######################################################################
#                    General infos on the datasets                    #
#######################################################################

# Metadata
MD = DataFrame(name=String[], 
                     source=String[],   # Useless for now… but could be useful at some point
                     n=Int[],
                     d=Int[],
                     med_pairwise_Euclidean=Float64[], 
                     med_pairwise_Euclidean_standardized=Float64[], )
push!(MD, ["fried",               "OpenML", 40768,  10,1.27,4.40])
push!(MD, ["stock",               "OpenML", 59049,  9, 30.82, 4.13])
push!(MD, ["mlr_knn_rng",         "OpenML", 111753, 10, 10280.0, 3.57])
push!(MD, ["methane",             "OpenML", 9199930,32, 91.54, 6.69])
push!(MD, ["diamonds",            "OpenML", 53940,  26, 4.4, 6.89])
push!(MD, ["protein",             "OpenML", 45730,  9, 480978.0, 3.046])
push!(MD, ["house_8L",            "OpenML", 22784,  8, 889.6, 4.15])
push!(MD, ["sulfur",              "OpenML", 10081,  6, 0.571, 3.14])
push!(MD, ["elevators",           "OpenML", 16599,  18, 267.7, 4.994])
push!(MD, ["sarcos",              "OpenML", 44484,  21, 26.15, 6.063])
push!(MD, ["year_prediction_msd", "UCI",    515345, 90, 3069.3, 10.95])

function MD_row(name) 
    r = findfirst(==(name), MD.name)
    @assert ~isnothing(r) "Unrecognized dataset '$(name)'. Supported values are: $(MD.name)."
    return r
end
median_pairwise_Euclidean(name) = MD.med_pairwise_Euclidean[MD_row(name)]
median_pairwise_Euclidean_standardized(name) = MD.med_pairwise_Euclidean_standardized[MD_row(name)]
get_n(name) = MD.n[MD_row(name)]
get_d(name) = MD.d[MD_row(name)]
get_source(name) = MD.source[MD_row(name)]

#######################################################################
#                            Preprocessing                            #
#######################################################################

function custom_coerce(T, coercion_rules = Dict();)
    if ~isempty(coercion_rules)
        coerce!(T, coercion_rules...)
    end
    # Automatically coerce to Continuous, using OneHotEncoder if needed
    encoder = ContinuousEncoder()
    W = MLJ.transform(fit!(machine(encoder, T)), T)
    return W
end

function standardize(df, resp_var;
        standardize_X = true,
        standardize_y = true)

    s = if standardize_X && standardize_y
        Standardizer()
    elseif standardize_X
        Standardizer(features=[resp_var], ignore=true)
    elseif standardize_y
        Standardizer(features=[resp_var])
    else nothing end
    return if ~isnothing(s)
        MLJ.transform(fit!(machine(s, df)), df)
    else
        df
    end
end

#######################################################################
#                      Loading and constructors                       #
#######################################################################

function load_dataset(T, name; kwargs...)
    if name in OML.name 
        return load_openml_dataset(T, name; kwargs...)
    elseif (name in keys(UCI)) 
        @assert isdefined(Main, :UCIData) "The `UCIData` module must be loaded to use UCI datasets."
        return load_uci_dataset(T, name; kwargs...)
    else
        @error "Unknown dataset '$name'. Supported datasets are: $(OML.name) (from OpenML), $(keys(UCI)) (from UCI)."
    end
end
load_dataset(name; kwargs...) = load_dataset(Matrix, name; kwargs...)

function load_dataset(::Type{Matrix}, name; transpose_X=true, kwargs...)
    X, y = load_dataset(DataFrame, name; kwargs...)
    if transpose_X
        return transpose(Matrix(X)), y
    else
        return Matrix(X), y
    end
end

